package be.iminds.ilabt.jfed.experimenter_gui.ui;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


public class ProcessHelper {
    private static final Logger LOG = LoggerFactory.getLogger(ProcessHelper.class);
    private List<Process> processes;


    public ProcessHelper() {
        processes = new ArrayList<Process>();

    }

    public Process startNewJavaProcess( final String mainClass, final String[] arguments) throws IOException {

        return startNewJavaProcess("-cp \"" + System.getProperty("java.class.path") + "\"", mainClass, arguments);
    }

    public Process startNewJavaProcess(final String optionsAsString, final String mainClass, final String[] arguments)
            throws IOException {

        ProcessBuilder processBuilder = createProcess(optionsAsString, mainClass, arguments);
        Process process = processBuilder.start();
        processes.add(process);
        LOG.debug("Process with main-class '{}' and arguments '{}' has started",
                mainClass, Arrays.stream(arguments).collect(Collectors.joining(", ")));
        return process;
    }


    private ProcessBuilder createProcess(final String optionsAsString, final String mainClass, final String[] arguments) {
        String jvm = System.getProperty("java.home") + File.separator + "bin" + File.separator + "java";
        String classpath = System.getProperty("java.class.path");

        String[] options = optionsAsString.split(" ");
        List<String> command = new ArrayList<>();
        command.add(jvm);
        command.addAll(Arrays.asList(options));
        command.add(mainClass);
        command.addAll(Arrays.asList(arguments));

        ProcessBuilder processBuilder = new ProcessBuilder(command);
        Map<String, String> environment = processBuilder.environment();
        environment.put("CLASSPATH", classpath);
        return processBuilder;
    }


    public void killProcess(final Process process) {
        process.destroy();
    }

    /**
     * Kill all processes.
     */
    public void shutdown() {
        LOG.debug("Killing {} processes.", processes.size());
        processes.forEach(this::killProcess);
    }


}